/*
 * This file is part of the CorpCor suite.
 * 
 * Copyright (C) 2012 by Instytut Podstaw Informatyki Polskiej
 * Akademii Nauk (IPI PAN; Institute of Computer Science, Polish
 * Academy of Sciences; cf. www.ipipan.waw.pl).  All rights reserved.
 * 
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file gpl.txt included in the packaging
 * of this file.  (See http://www.gnu.org/licenses/translations.html for
 * unofficial translations.)
 * 
 * A commercial license is available from IPI PAN (contact 
 * ipi@ipipan.waw.pl for more information).  Licensees holding a valid 
 * commercial license from IPI PAN may use this file in accordance with 
 * that license.
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING
 * THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.
 */
package pl.waw.ipipan.corpcor.server.pq.client.tickets;

import pl.waw.ipipan.corpcor.server.pq.client.util.SimpleFuture;

public abstract class PQCRequestTicket<V> extends SimpleFuture<V> {

    private final TicketType type;
    private boolean failed;

    public static enum TicketType {
        QUERY, METADATA, WIDECONTEXT;
    }

    public PQCRequestTicket(TicketType type) {
        this.type = type;
    }

    public final TicketType getType() {
        return this.type;
    }

    public final synchronized void setFailedResult() {
        super.setResult(createFailedResult());
        this.failed = true;
    }

    protected abstract V createFailedResult();

    public final synchronized boolean isFailed() {
        return this.failed;
    }
    
    public abstract PQCRequestTicket<V> createClone();
}
